@@ -1,9 +1,10 @@
 import os
+import operator
 
 from django.contrib.auth.models import User
 from django.db import models
 from django.utils.translation import ugettext as _
-from django.utils.text import Truncator
+from django.utils.text import Truncator, slugify
 
 
 class TimestampedModel(models.Model):
@@ -14,33 +15,54 @@ class TimestampedModel(models.Model):
         abstract = True
 
 
+class CategoryManager(models.Manager):
+    def children_of(self, category, categories=None, acc=None):
+        if acc is None:
+            acc = []
+
+        if categories is None:
+            categories = self.all()
+
+        children = filter(lambda c: c.parent == category, categories)
+        for child in children:
+            acc.extend(self.children_of(child, categories, acc))
+
+        acc.extend(children)
+
+        return acc
+
+
 class Category(models.Model):
     title = models.CharField(_('title'), max_length=100)
     parent = models.ForeignKey('self', blank=True, null=True)
+    slug = models.SlugField()
+
+    objects = CategoryManager()
 
     class Meta:
-        ordering = ('title', )
         verbose_name = u'category'
         verbose_name_plural = u'categories'
 
-    @property
-    def full_title(self):
-        """
-            Returns a '>' separated list of the current category's parents
-            title + the current category's title.
-        """
-        parents = [category.title for category in self.parents()]
-
-        if parents:
-            return u"{parents} > {title}".format(
-                parents=" > ".join(parents),
-                title=self.title,
-            )
-        else:
-            return self.title
+    def save(self, *args, **kwargs):
+        self.slug = self._generate_slug()
+        super(Category, self).save(*args, **kwargs)
 
     def __unicode__(self):
-        return self.full_title
+        return " > ".join([category.title for category in self.hierarchy()])
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ('hermes_category_post_list', (), {
+            'slug': self.slug,
+        })
+
+    def _generate_slug(self):
+        return "/".join([slugify(category.title) for category in self.hierarchy()]).lower()
+
+    @property
+    def is_root(self):
+        """ Returns True if this category has no parent. """
+        return self.parent == None
 
     def parents(self):
         """ Returns a list of all the current category's parents."""
@@ -54,24 +76,37 @@ class Category(models.Model):
             parents.append(category.parent)
             category = category.parent
 
-        return parents
+        return parents[::-1]
 
-    def is_root(self):
-        """ Returns True if this category has no parent. """
-        return self.parent == None
+    def hierarchy(self):
+        return self.parents() + [self]
 
     def root_parent(self, category=None):
-        """ Gets the topmost parent of the current category. """
-        if not category:
-            category = self
-
-        if category.is_root():
-            return category
-        else:
-            return self.root_parent(category.parent)
+        """ Returns the topmost parent of the current category. """
+        return filter(lambda c: c.is_root, self.hierarchy())[0]
 
 
 class PostQuerySet(models.query.QuerySet):
+    def in_category(self, category_slug):
+        category = Category.objects.get(slug=category_slug)
+        children = Category.objects.children_of(category)
+
+        return self.filter(category__in=[category] + children)
+
+    def created_on(self, year=None, month=None, day=None):
+        clauses = []
+
+        if year:
+            clauses.append(models.Q(created_on__year=year))
+
+        if month:
+            clauses.append(models.Q(created_on__month=month))
+
+        if day:
+            clauses.append(models.Q(created_on__day=day))
+
+        return self.filter(reduce(operator.__and__, clauses))
+
     def recent(self, limit=None):
         queryset = self.all()
         if limit:
